@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Program Utilities
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2001-2003 Hubert Figuiere
@@ -43,6 +45,9 @@ class EV_CocoaToolbar;
 {
 	EV_CocoaToolbar*	_xap;
 }
+- (void)setColor:(XAP_Toolbar_Id)tlbrid;
+- (IBAction)aColor_FG:(id)sender;
+- (IBAction)aColor_BG:(id)sender;
 - (id)toolbarSelected:(id)sender;
 - (void)setXAPOwner:(EV_CocoaToolbar*)owner;
 @end
@@ -56,7 +61,7 @@ public:
 	
 	virtual ~EV_CocoaToolbar(void);
 
-	bool toolbarEvent(XAP_Toolbar_Id tlbrid, UT_UCSChar * pData, UT_uint32 dataLength);
+	bool toolbarEvent(XAP_Toolbar_Id tlbrid, const UT_UCSChar * pData, UT_uint32 dataLength);
 	virtual bool synthesize(void);
 	bool bindListenerToView(AV_View * pView);
 	virtual bool refreshToolbar(AV_View * pView, AV_ChangeMask mask);
@@ -69,13 +74,12 @@ public:
 	virtual void hide(void);
 
 	static	float	getButtonWidth (void)
-					{ return 34.0f; };
+					{ return 32.0f; };
 	static	float	getButtonHeight (void)
-					{ return 34.0f; };
-	static	float	getButtonSpace (void) 
-					{ return 1.0f; };
+					{ return 32.0f; };
+
 	static float  getToolbarHeight(void)
-					{ return getButtonHeight() + (getButtonSpace() * 2); };
+					{ return getButtonHeight(); };
 	
 	NSView*	_getToolbarView(void) const 
 				{ return m_wToolbar; };
