@@ -1,6 +1,8 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
- * Copyright (C) 2001-2002 Hubert Figuiere
+ * Copyright (C) 2001-2004 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +39,9 @@
 #import "gr_CocoaGraphics.h"
 #import "xap_App.h"
 #import "xap_CocoaApp.h"
+#import "xap_CocoaAppController.h"
 #import "xap_CocoaFrameImpl.h"
+#import "xap_CocoaTextView.h"
 #import "xap_CocoaTimer.h"
 #import "xap_FrameImpl.h"
 #import "xap_Frame.h"
@@ -170,7 +174,7 @@ void XAP_CocoaFrameImpl::_initialize()
    	// get a handle to our keyboard binding mechanism
 	// and to our mouse binding mechanism.
 
-	EV_EditEventMapper * pEEM = getFrame()->getEditEventMapper();
+	EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 	UT_ASSERT(pEEM);
 
 	m_pKeyboard = new ev_CocoaKeyboard(pEEM);
@@ -193,15 +197,23 @@ void XAP_CocoaFrameImpl::_initialize()
 	}
 }
 
+void XAP_CocoaFrameImpl::notifyViewChanged(AV_View * pView) // called from XAP_Frame::setView(pView)
+{
+	if (XAP_Frame * pFrame = getFrame())
+		{
+			XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+			[pController resetCurrentView:pView inFrame:pFrame];
+		}
+}
+
 UT_sint32 XAP_CocoaFrameImpl::_setInputMode(const char * szName)
 {
-	XAP_Frame*	pFrame = getFrame();
-	UT_sint32 result = pFrame->XAP_Frame::setInputMode(szName);
+	UT_sint32 result = XAP_App::getApp()->setInputMode(szName);
 	if (result == 1)
 	{
 		// if it actually changed we need to update keyboard and mouse
 
-		EV_EditEventMapper * pEEM = pFrame->getEditEventMapper();
+		EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 		UT_ASSERT(pEEM);
 
 		m_pKeyboard->setEditEventMap(pEEM);
@@ -230,11 +242,13 @@ XAP_DialogFactory * XAP_CocoaFrameImpl::_getDialogFactory(void)
 
 void XAP_CocoaFrameImpl::_nullUpdate() const
 {
-#if 0
-  gtk_main_iteration ();
-#endif
+	/* FIXME: do this more propperly if there is any chance...*/
+	[[m_frameController window] displayIfNeeded];
 }
 
+static UT_uint32	s_iNewFrameOffsetX = 0;
+static UT_uint32	s_iNewFrameOffsetY = 0;
+
 void XAP_CocoaFrameImpl::_createTopLevelWindow(void)
 {
 	// create a top-level window for us.
@@ -274,13 +288,20 @@ void XAP_CocoaFrameImpl::_createTopLevelWindow(void)
 
 	if (f & XAP_CocoaApp::GEOMETRY_FLAG_SIZE)
 	{
-		NSRect windowFrame;
 		NSRect screenFrame = [[NSScreen mainScreen] visibleFrame];
 
-		windowFrame.size.width = UT_MIN(screenFrame.size.width - 30, 813);
-		windowFrame.size.height = UT_MIN(screenFrame.size.height - 100, 836);
+		screenFrame.origin.x    += s_iNewFrameOffsetX;
+		screenFrame.size.height -= s_iNewFrameOffsetY;
+
+		s_iNewFrameOffsetX = (s_iNewFrameOffsetX <= 128) ? (s_iNewFrameOffsetX + 32) : 0;
+		s_iNewFrameOffsetY = (s_iNewFrameOffsetY <  128) ? (s_iNewFrameOffsetY + 32) : 0;
+
+		NSRect windowFrame;
+		windowFrame.size.width  = UT_MIN(screenFrame.size.width - 30, 813);
+		windowFrame.size.height = UT_MIN(screenFrame.size.height, 836);
 		windowFrame.origin.x = screenFrame.origin.x;
-		windowFrame.origin.y = screenFrame.origin.y;
+		windowFrame.origin.y = screenFrame.size.height - windowFrame.size.height;
+
 		[theWindow setFrame:windowFrame display:YES];
 	}
 
@@ -302,35 +323,6 @@ void XAP_CocoaFrameImpl::_createTopLevelWindow(void)
 	return;
 }
 
-/*!
- * This code is used by the dynamic menu API to rebuild the menus after a
- * a change in the menu structure.
- */
-void XAP_CocoaFrameImpl::rebuildMenus(void)
-{
-	UT_ASSERT_NOT_REACHED();
-#if 0
-//
-// Destroy the old menu bar
-//
-	m_pCocoaMenu->destroy();
-//
-// Delete the old class
-//
-	DELETEP(m_pCocoaMenu);
-//
-// Build a new one.
-//
-	m_pCocoaMenu = new EV_CocoaMenuBar(m_pCocoaApp,(AP_CocoaFrame*)getFrame(),
-									 m_szMenuLayoutName,
-									 m_szMenuLabelSetName);
-	UT_ASSERT(m_pCocoaMenu);
-	bool bResult = m_pCocoaMenu->rebuildMenuBar();
-	UT_ASSERT(bResult);
-#endif
-}
-
-
 /*!
  * This code is used by the dynamic toolbar API to rebuild a toolbar after a
  * a change in the toolbar structure.
@@ -359,7 +351,7 @@ void XAP_CocoaFrameImpl::_rebuildToolbar(UT_uint32 ibar)
 	pToolbar = _newToolbar(m_pCocoaApp, pFrame, szTBName,
 						   (const char *) m_szToolbarLabelSetName);
 	static_cast<EV_CocoaToolbar *>(pToolbar)->rebuildToolbar(oldpos);
-	m_vecToolbars.setNthItem(ibar, (void *) pToolbar, NULL);
+	m_vecToolbars.setNthItem(ibar, pToolbar, NULL);
 //
 // Refill the framedata pointers
 //
@@ -385,25 +377,12 @@ bool XAP_CocoaFrameImpl::_raise()
 
 bool XAP_CocoaFrameImpl::_show()
 {
-	UT_DEBUGMSG (("XAP_CocoaFrame::raise()\n"));
+	UT_DEBUGMSG (("XAP_CocoaFrame::show()\n"));
 	[[m_frameController window] makeKeyAndOrderFront:m_frameController];
 	[[NSNotificationCenter defaultCenter] postNotificationName:XAP_FrameNeedToolbar object:m_frameController];
 	return true;
 }
 
-bool XAP_CocoaFrameImpl::_openURL(const char * szURL)
-{  
-	NSString* str = [[NSString alloc] initWithUTF8String:szURL];
-	NSURL *URL = [[NSURL alloc] initWithString:str];		
-	
-	NSWorkspace * space = [NSWorkspace sharedWorkspace];
-	[space openURL:URL];
-
-	[URL release];
-	[str release];
-	
-	return true;
-}
 
 UT_RGBColor XAP_CocoaFrameImpl::getColorSelBackground () const
 {
@@ -426,23 +405,19 @@ bool XAP_CocoaFrameImpl::_updateTitle()
 		return false;
 	}
 
-	char buf[256];
-	buf[0] = 0;
-
-	const char * szAppName = m_pCocoaApp->getApplicationTitleForTitleBar();
-
-	int len = 256 - strlen(szAppName) - 4;
-	
-	const char * szTitle = getFrame()->getTitle(len);
-
-	//sprintf(buf, "%s - %s", szTitle, szAppName);
-	/* TODO discard this sprintf and you NSString features instead */
 	NSWindow * theWindow = [m_frameController window];
 	UT_ASSERT (theWindow);
-	NSString * str = [[NSString alloc] initWithUTF8String:szTitle];
-	[theWindow setTitleWithRepresentedFilename:str];
-	[str release];
-	
+	if (theWindow)
+		{
+			const char * szTitle    = getFrame()->getNonDecoratedTitle();
+			const char * szFilename = getFrame()->getFilename();
+
+			[theWindow setTitle:[NSString stringWithUTF8String:szTitle]];
+			if (szFilename)
+				[theWindow setRepresentedFilename:[NSString stringWithUTF8String:szFilename]];
+
+			[theWindow setDocumentEdited:(getFrame()->isDirty() ? YES : NO)];
+		}
 	return true;
 }
 
@@ -450,56 +425,21 @@ bool XAP_CocoaFrameImpl::_updateTitle()
 bool XAP_CocoaFrameImpl::_runModalContextMenu(AV_View * /* pView */, const char * szMenuName,
 										   UT_sint32 x, UT_sint32 y)
 {
-#if 0
 	bool bResult = true;
 
 	UT_ASSERT(!m_pCocoaPopup);
 
-	m_pCocoaPopup = new EV_CocoaMenuPopup(m_pCocoaApp,this,szMenuName,m_szMenuLabelSetName);
-	if (m_pCocoaPopup && m_pCocoaPopup->synthesizeMenuPopup())
-	{
-		// the popup will steal the mouse and so we won't get the
-		// button_release_event and we won't know to release our
-		// grab.  so let's do it here.  (when raised from a keyboard
-		// context menu, we may not have a grab, but that should be ok.
-
-		GtkWidget * w = gtk_grab_get_current();
-		if (w)
-		{
-			//UT_DEBUGMSG(("Ungrabbing mouse [before popup].\n"));
-			gtk_grab_remove(w);
-		}
-
-		translateDocumentToScreen(x,y);
-
-		UT_DEBUGMSG(("ContextMenu: %s at [%d,%d]\n",szMenuName,x,y));
-		UT_Point pt;
-		pt.x = x;
-		pt.y = y;
-//
-// OK lets not immediately drop the menu if the user releases the mouse button.
-// From the gtk FAQ.
-//
-		GdkEvent * event = gtk_get_current_event();
-		GdkEventButton *bevent = (GdkEventButton *) event; 
-		gtk_menu_popup(GTK_MENU(m_pCocoaPopup->getMenuHandle()), NULL, NULL,
-					   s_gtkMenuPositionFunc, &pt, bevent->button, bevent->time);
-
-		// We run this menu synchronously, since GTK doesn't.
-		// Popup menus have a special "unmap" function to call
-		// gtk_main_quit() when they're done.
-		gtk_main();
+	m_pCocoaPopup = new EV_CocoaMenuPopup(m_pCocoaApp,szMenuName,m_szMenuLabelSetName);
+	if (m_pCocoaPopup && m_pCocoaPopup->synthesizeMenuPopup()) {
+		NSEvent *evt = [NSApp currentEvent];
+		[NSMenu popUpContextMenu:m_pCocoaPopup->getMenuHandle() withEvent:evt forView:[m_frameController getMainView]];
 	}
-	XAP_Frame * pFrame = (XAP_Frame *) this;
-	if (pFrame->getCurrentView())
-	{
+	XAP_Frame * pFrame = (XAP_Frame *)getFrame();
+	if (pFrame->getCurrentView()) {
 		pFrame->getCurrentView()->focusChange( AV_FOCUS_HERE);
 	}
 	DELETEP(m_pCocoaPopup);
 	return bResult;
-#endif
-	UT_ASSERT(UT_NOT_IMPLEMENTED);
-	return false;
 }
 
 void XAP_CocoaFrameImpl::setTimeOfLastEvent(NSTimeInterval timestamp)
@@ -538,6 +478,28 @@ void XAP_CocoaFrameImpl::_setController (XAP_CocoaFrameController * ctrl)
 	m_frameController = ctrl; 
 }
 
+void XAP_CocoaFrameImpl::setToolbarRect(const NSRect &r)
+{
+	UT_uint32 frameCount = XAP_App::getApp()->getFrameCount();
+	UT_uint32 i;
+	
+	for (i = 0; i < frameCount; i++) {
+		XAP_CocoaFrameImpl* impl = dynamic_cast<XAP_CocoaFrameImpl*>(XAP_App::getApp()->getFrame(i)->getFrameImpl());
+		XAP_CocoaFrameController* ctrl  = impl->_getController();
+		NSRect frame = [[ctrl window] frame];
+		if (NSIntersectsRect(frame, r)) {
+			if (frame.origin.y + frame.size.height > r.origin.y) {
+				UT_DEBUGMSG(("original frame is %f %f %f %f\n", frame.origin.x, frame.origin.y,
+									frame.size.width, frame.size.height));
+				frame.size.height = r.origin.y - frame.origin.y;
+				[[ctrl window] setFrame:frame display:YES];
+				UT_DEBUGMSG(("resized frame is %f %f %f %f\n", frame.origin.x, frame.origin.y,
+									frame.size.width, frame.size.height));
+			}
+		}
+	}
+}
+
 
 /* Objective C section */
 
@@ -564,7 +526,14 @@ void XAP_CocoaFrameImpl::_setController (XAP_CocoaFrameController * ctrl)
 		if (pEM->Fn(m_frame->getFrame()->getCurrentView(),NULL))
 		{
 			// returning YES means close the window
+			// but first let's warn the app controller not to assume our existence
 			
+			XAP_Frame * pFrame = m_frame->getFrame ();
+			AV_View * pView = pFrame->getCurrentView();
+
+			XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+			[pController unsetCurrentView:pView inFrame:pFrame];
+
 			return YES;
 		}
 	}
@@ -573,11 +542,9 @@ void XAP_CocoaFrameImpl::_setController (XAP_CocoaFrameController * ctrl)
 	return NO;
 }
 
-
-
 - (void)keyDown:(NSEvent *)theEvent
 {
-	
+	xxx_UT_DEBUGMSG(("keyDown in window '%s'\n", [[[self window] title] UTF8String]));
 	[m_textView interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
 #if 0
 	XAP_Frame * pFrame = m_frame->getFrame();
@@ -593,18 +560,32 @@ void XAP_CocoaFrameImpl::_setController (XAP_CocoaFrameController * ctrl)
 
 - (void)windowDidBecomeKey:(NSNotification *)aNotification
 {
+	UT_DEBUGMSG(("windowDidBecomeKey: '%s'\n", [[[self window] title] UTF8String]));
+
+	XAP_Frame * pFrame = m_frame->getFrame ();
+	AV_View * pView = pFrame->getCurrentView();
+
+	XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+	[pController setCurrentView:pView inFrame:pFrame];
+
 	[[NSNotificationCenter defaultCenter] postNotificationName:XAP_CocoaFrameImpl::XAP_FrameNeedToolbar 
 			object:self];
 }
 
 - (void)windowDidExpose:(NSNotification *)aNotification
 {
+	UT_DEBUGMSG(("windowDidExpose:\n"));
 //	[[NSNotificationCenter defaultCenter] postNotificationName:XAP_CocoaFrameImpl::XAP_FrameNeedToolbar 
 //			object:self];
 }
 
 - (void)windowDidResignKey:(NSNotification *)aNotification
 {
+	UT_DEBUGMSG(("windowDidResignKey: '%s'\n", [[[self window] title] UTF8String]));
+
+	XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+	[pController setCurrentView:0 inFrame:0];
+
 //	[[NSNotificationCenter defaultCenter] postNotificationName:XAP_CocoaFrameImpl::XAP_FrameReleaseToolbar 
 //			object:self];
 }
@@ -659,7 +640,7 @@ void XAP_CocoaFrameImpl::_setController (XAP_CocoaFrameController * ctrl)
 - (NSArray*)getToolbars
 {
 	NSMutableArray*	array = [NSMutableArray array];
-	const UT_Vector & toolbars = m_frame->_getToolbars();
+	const UT_GenericVector<EV_Toolbar*> & toolbars = m_frame->_getToolbars();
 	UT_uint32 count = toolbars.getItemCount();
 	for (UT_uint32 i = 0; i < count; i++) {
 		const EV_CocoaToolbar* tlbr = static_cast<const EV_CocoaToolbar*>(toolbars[i]);
