@@ -2,7 +2,7 @@
 
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
- * Copyright (C) 2001 Hubert Figuiere
+ * Copyright (C) 2001, 2003 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,8 +20,6 @@
  * 02111-1307, USA.
  */
  
-#import <Cocoa/Cocoa.h>
-
 #include <stdio.h>
 #include <string.h>
 
@@ -32,6 +30,98 @@
 
 #include "xap_CocoaToolbar_Icons.h"
 
+static NSPoint s_ButtonOnPoint[12] = {
+	{	 1.0f,	 8.0f	},
+	{	 1.0f,	 1.0f	},
+	{	 8.0f,	 1.0f	},
+	{	24.0f,	 1.0f	},
+	{	31.0f,	 1.0f	},
+	{	31.0f,	 8.0f	},
+	{	31.0f,	24.0f	},
+	{	31.0f,	31.0f	},
+	{	24.0f,	31.0f	},
+	{	 8.0f,	31.0f	},
+	{	 1.0f,	31.0f	},
+	{	 1.0f,	24.0f	}
+};
+
+static NSPoint s_ButtonMenuPoint[3] = {
+	{	30.0f,	25.0f	},
+	{	24.0f,	25.0f	},
+	{	27.0f,	30.0f	}
+};
+
+@implementation XAP_CocoaToolbarButton
+
+- (id)initWithFrame:(NSRect)frameRect
+{
+	if (self = [super initWithFrame:frameRect])
+		{
+			m_menu = 0;
+			m_controller = 0;
+		}
+	return self;
+}
+
+- (void)setMenu:(NSMenu *)menu withController:(id <XAP_CocoaButtonController>)controller
+{
+	m_menu = menu;
+	m_controller = controller;
+}
+
+- (void)mouseDown:(NSEvent *)theEvent
+{
+	if (m_menu && [self isEnabled])
+		{
+			if (m_controller)
+				{
+					[m_controller menuWillActivate:m_menu forButton:self];
+				}
+			[NSMenu popUpContextMenu:m_menu withEvent:theEvent forView:self];
+		}
+	else
+		{
+			[super mouseDown:theEvent];
+		}
+}
+
+- (void)drawRect:(NSRect)aRect
+{
+	if ([self state] == NSOnState)
+		{
+			[[NSColor colorWithCalibratedWhite:0.0f alpha:0.25] set];
+
+			NSBezierPath * path = [NSBezierPath bezierPath];
+
+			[path  moveToPoint:s_ButtonOnPoint[ 0]];
+			[path curveToPoint:s_ButtonOnPoint[ 2] controlPoint1:s_ButtonOnPoint[ 1] controlPoint2:s_ButtonOnPoint[ 1]];
+			[path  lineToPoint:s_ButtonOnPoint[ 3]];
+			[path curveToPoint:s_ButtonOnPoint[ 5] controlPoint1:s_ButtonOnPoint[ 4] controlPoint2:s_ButtonOnPoint[ 4]];
+			[path  lineToPoint:s_ButtonOnPoint[ 6]];
+			[path curveToPoint:s_ButtonOnPoint[ 8] controlPoint1:s_ButtonOnPoint[ 7] controlPoint2:s_ButtonOnPoint[ 7]];
+			[path  lineToPoint:s_ButtonOnPoint[ 9]];
+			[path curveToPoint:s_ButtonOnPoint[11] controlPoint1:s_ButtonOnPoint[10] controlPoint2:s_ButtonOnPoint[10]];
+			[path closePath];
+			[path fill];
+		}
+	[super drawRect:aRect];
+
+	if (m_menu)
+		{
+			[[NSColor blackColor] set];
+
+			NSBezierPath * path = [NSBezierPath bezierPath];
+
+			[path moveToPoint:s_ButtonMenuPoint[0]];
+			[path lineToPoint:s_ButtonMenuPoint[1]];
+			[path lineToPoint:s_ButtonMenuPoint[2]];
+			[path closePath];
+			[path fill];
+		}
+}
+
+@end
+
 AP_CocoaToolbar_Icons::AP_CocoaToolbar_Icons(void)
 {
 }
@@ -56,17 +146,38 @@ typedef struct _my_argb {
 	\param szIconName the name of the icon
 	\return the newly allocated NSImage [autoreleased]
  */
-NSImage* AP_CocoaToolbar_Icons::getPixmapForIcon(const char * szIconName)
+NSImage* AP_CocoaToolbar_Icons::getPixmapForIcon(const char * szIconID)
 {
-	UT_ASSERT(szIconName && *szIconName);
+	UT_ASSERT(szIconID && *szIconID);
 	NSImage *pixmap = nil;
 
+#if 1
+	//begin cocoa icon loading
+	const char * szIconName;
+	if (AP_Toolbar_Icons::_findIconNameForID(szIconID, &szIconName)) {
+		char *s = strdup(szIconName);
+		char *tmp = strstr(s, "_xpm");
+		if (tmp) {
+			*tmp = 0;
+		}
+		NSString * str = [[NSString alloc] initWithUTF8String:s];
+		pixmap = [NSImage imageNamed:str];
+		[str release];
+		FREEP(s);
+		UT_ASSERT(pixmap);
+		return pixmap;
+	}
+	else {
+		UT_ASSERT_NOT_REACHED();
+		return nil;
+	}
+#else
 	UT_uint32 width, height, nrColors, charsPerPixel;
 	
 	const char ** pIconData = NULL;
 	UT_uint32 sizeofIconData = 0;		// number of cells in the array
 	
-	bool bFound = _findIconDataByName(szIconName, &pIconData, &sizeofIconData);
+	bool bFound = _findIconDataByName(szIconID, &pIconData, &sizeofIconData);
 	if (!bFound)
 		return nil;
 
@@ -170,5 +281,6 @@ NSImage* AP_CocoaToolbar_Icons::getPixmapForIcon(const char * szIconName)
 	UT_ASSERT (pixmap);	
 
 	return [pixmap autorelease];
+#endif
 }
 
