@@ -484,22 +484,18 @@ void createLabelAccelerators( GtkWidget *widget )
 /****************************************************************/
 
 // in ap_editmethods.cpp
-extern bool helpLocalizeAndOpenURL(XAP_Frame * pFrame, bool bLocal, const char* pathBeforeLang, const char* pathAfterLang);
+extern bool helpLocalizeAndOpenURL(const char* pathBeforeLang, const char* pathAfterLang, const char *remoteURLbase);
 
 static void sDoHelp ( XAP_Dialog * pDlg )
 {
-	// get any frame to open up a URL, doesn't matter which one
-	XAP_App   * pApp   = XAP_App::getApp () ;
-	XAP_Frame * pFrame = pApp->getLastFocussedFrame () ;
-	
 	// should always be valid, but just in case...
-	if (!pDlg || !pFrame)
+	if (!pDlg)
 		return;
 
 	// open the url
 	if ( pDlg->getHelpUrl().size () > 0 )
     {
-		helpLocalizeAndOpenURL ( pFrame, true, "AbiWord/help", pDlg->getHelpUrl().c_str() ) ;
+		helpLocalizeAndOpenURL ("AbiWord/help", pDlg->getHelpUrl().c_str(), NULL );
     }
 	else
     {
@@ -555,6 +551,12 @@ static void help_button_cb (GObject * button, XAP_Dialog * pDlg)
 
 static void sAddHelpButton (GtkDialog * me, XAP_Dialog * pDlg)
 {
+  // prevent help button from being added twice
+  gint has_button = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (me), "has-help-button"));
+
+  if (has_button)
+	  return;
+
   if (pDlg && pDlg->getHelpUrl().size () > 0) {
     GtkWidget * button = gtk_button_new_from_stock (GTK_STOCK_HELP);
 
@@ -565,6 +567,8 @@ static void sAddHelpButton (GtkDialog * me, XAP_Dialog * pDlg)
     g_signal_connect (G_OBJECT (button), "clicked",
 		      G_CALLBACK(help_button_cb), pDlg);
     gtk_widget_show (button);
+
+    g_object_set_data (G_OBJECT (me), "has-help-button", GINT_TO_POINTER (1));
   }
 }
 
@@ -613,14 +617,11 @@ void abiSetupModalDialog(GtkDialog * me, XAP_Frame *pFrame, XAP_Dialog * pDlg, g
 	
 	sAddHelpButton (me, pDlg);
 
-	// show the window
-	gtk_widget_show ( GTK_WIDGET(me) ) ;
-	
-	// grab the dialog
-	//gtk_grab_add ( GTK_WIDGET(me) ) ;
-	
 	// and make it modal
 	gtk_window_set_modal ( GTK_WINDOW(me), TRUE ) ;
+
+	// show the window
+	gtk_widget_show ( GTK_WIDGET(me) ) ;
 }
 
 gint abiRunModalDialog(GtkDialog * me, bool destroyDialog, AtkRole role)
@@ -692,17 +693,14 @@ void abiSetupModelessDialog(GtkDialog * me, XAP_Frame * pFrame, XAP_Dialog * pDl
 	gtk_dialog_set_default_response ( me, dfl_response ) ;
 
 	sAddHelpButton (me, pDlg);
-	
-	// show the window
-	gtk_widget_show ( GTK_WIDGET(me) ) ;
-	
-	// grab the dialog
-	//gtk_grab_add ( GTK_WIDGET(me) ) ;
-	
+
 	// and mark it as modeless
 	gtk_window_set_modal ( GTK_WINDOW(me), FALSE ) ;
 
 	atk_object_set_role (gtk_widget_get_accessible (GTK_WIDGET (me)), ATK_ROLE_ALERT);
+	
+	// show the window
+	gtk_widget_show ( GTK_WIDGET(me) ) ;
 }
 
 /*!
@@ -771,6 +769,28 @@ GladeXML * abiDialogNewFromXML(const char * glade_file)
 	return xml;
 }
 	
+
+/*!
+ * Returns a GtkMenu with items having label fetched from UTF-8 CStr 
+ * from UT_Vector.
+ * All menu item will have the index of the item stored in its user-data
+ */
+GtkWidget * abiGtkMenuFromCStrVector(const UT_GenericVector<const char*> & vec, GCallback cb, gpointer data)
+{
+	UT_uint32 i;
+	GtkWidget * menu = gtk_menu_new();
+	
+	for (i = 0; i < vec.getItemCount(); i++) {
+		GtkWidget * menuItem = gtk_menu_item_new_with_label(vec[i]);
+		g_object_set_data(G_OBJECT(menuItem), "user_data", reinterpret_cast<void*>(i));
+		g_signal_connect(G_OBJECT(menuItem), "activate", G_CALLBACK(cb), data);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
+	}
+	gtk_widget_show_all(menu);
+	return menu;
+}
+
+
 /*!
  * Set the title of a gtk dialog
  */
@@ -840,33 +860,43 @@ void abiDestroyWidget(GtkWidget * me)
 void localizeLabel(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id id)
 {
 	XML_Char * unixstr = NULL;	// used for conversions
-	
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(id).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_label_set_text (GTK_LABEL(widget), unixstr);
 	FREEP(unixstr);	
 }
 
+void convertMnemonics(XML_Char * s)
+{
+	UT_ASSERT(s);
+
+	for (UT_uint32 i = 0; s[i] != 0; i++) 
+	{
+		if ( s[i] == '&' ) {
+			if (i > 0 && s[i-1] == '\\')
+			{
+				s[i-1] = '&';
+				strcpy( &s[i], &s[i+1]);
+				i--;
+				}
+			else
+				s[i] = '_';
+		}
+	}
+}
+
 /*!
  * Localizes the label of a widget given the string id
  * Ampersands will be converted to underscores/mnemonics
  */
 void localizeLabelUnderline(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id id)
 {
-	XML_Char * newlbl = UT_strdup(pSS->getValueUTF8(id).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	XML_Char * newlbl = UT_strdup(s.utf8_str());
 	UT_ASSERT(newlbl);
-	for (UT_uint32 i = 0; newlbl[i] != 0; i++) 
-	{
-		if ( newlbl[i] == '&' ) {
-			if (i > 0 && newlbl[i-1] == '\\')
-			{
-				newlbl[i-1] = '&';
-				strcpy( &newlbl[i], &newlbl[i+1]);
-				i--;
-				}
-			else
-				newlbl[i] = '_';
-		}
-	}
+	convertMnemonics(newlbl);
 	gtk_label_set_text_with_mnemonic (GTK_LABEL(widget), newlbl);
 	FREEP(newlbl);	
 }
@@ -880,8 +910,9 @@ void localizeLabelUnderline(GtkWidget * widget, const XAP_StringSet * pSS, XAP_S
 void localizeLabelMarkup(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id id)
 {
 	XML_Char * unixstr = NULL;	// used for conversions
-	
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(id).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	UT_String markupStr(UT_String_sprintf(gtk_label_get_label (GTK_LABEL(widget)), unixstr));
 	gtk_label_set_markup (GTK_LABEL(widget), markupStr.c_str());
 	FREEP(unixstr);	
@@ -893,8 +924,9 @@ void localizeLabelMarkup(GtkWidget * widget, const XAP_StringSet * pSS, XAP_Stri
 void localizeButton(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id id)
 {
 	XML_Char * unixstr = NULL;	// used for conversions
-	
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(id).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_button_set_label (GTK_BUTTON(widget), unixstr);
 	FREEP(unixstr);	
 }
@@ -905,26 +937,42 @@ void localizeButton(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id
  */
 void localizeButtonUnderline(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id id)
 {
-	XML_Char * newlbl = UT_strdup(pSS->getValueUTF8(id).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	XML_Char * newlbl = UT_strdup(s.utf8_str());
 	UT_ASSERT(newlbl);
-	for (UT_uint32 i = 0; newlbl[i] != 0; i++) 
-	{
-		if ( newlbl[i] == '&' ) {
-			if (i > 0 && newlbl[i-1] == '\\')
-			{
-				newlbl[i-1] = '&';
-				strcpy( &newlbl[i], &newlbl[i+1]);
-				i--;
-				}
-			else
-				newlbl[i] = '_';
-		}
-	}
+	convertMnemonics(newlbl);
 	gtk_button_set_use_underline (GTK_BUTTON(widget), TRUE);
 	gtk_button_set_label (GTK_BUTTON(widget), newlbl);
 	FREEP(newlbl);	
 }
 
+/*!
+ * Localizes a button given the string id
+ * It formats its label using the current button label as a format
+ * string. It is assumed to be something like
+ * "<span size="larger">%s</span>".
+ * Note that in addition to doing markup, ampersands will be converted
+ * to underscores/mnemonic since this makes sense for buttons
+ */
+void localizeButtonMarkup(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id id)
+{
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	XML_Char * newlbl = UT_strdup(s.utf8_str());
+	UT_ASSERT(newlbl);
+	convertMnemonics(newlbl);
+	UT_String markupStr(UT_String_sprintf(gtk_button_get_label (GTK_BUTTON(widget)), newlbl));
+	gtk_button_set_use_underline (GTK_BUTTON(widget), TRUE);
+	gtk_button_set_label (GTK_BUTTON(widget), markupStr.c_str());
+
+	// by default, they don't like markup, so we teach them
+	GtkWidget * button_child = gtk_bin_get_child (GTK_BIN(widget));
+	if (GTK_IS_LABEL (button_child))
+		gtk_label_set_use_markup (GTK_LABEL(button_child), TRUE);
+
+	FREEP(newlbl);	
+}
 /*!
  * Localizes the label of a Menu widget given the string id
  */
@@ -932,8 +980,9 @@ void localizeMenu(GtkWidget * widget, const XAP_StringSet * pSS, XAP_String_Id i
 {
 //	UT_ASSERT(GTK_IS_MENU(widget));
 	XML_Char * unixstr = NULL;	// used for conversions
-	
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(id).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(id,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_menu_set_title (GTK_MENU(widget), unixstr);
 	FREEP(unixstr);	
 }
