@@ -19,9 +19,8 @@
  * 02111-1307, USA.
  */
 
-#include <gtk/gtk.h>
 #undef GTK_DISABLE_DEPRECATED
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,6 +41,9 @@
 #include "xap_Prefs.h"
 #include "ut_debugmsg.h"
 #include "ut_string_class.h"
+#if GTK_CHECK_VERSION(2,4,0)
+#include "ut_path.h"
+#endif
 
 #include "ut_png.h"
 #include "ut_svg.h"
@@ -51,6 +53,7 @@
 #include "fg_GraphicRaster.h"
 
 #include "gr_UnixImage.h"
+#include "gr_Painter.h"
 
 #include <sys/stat.h>
 
@@ -71,7 +74,11 @@ XAP_Dialog * XAP_UnixDialog_FileOpenSaveAs::static_constructor(XAP_DialogFactory
 
 XAP_UnixDialog_FileOpenSaveAs::XAP_UnixDialog_FileOpenSaveAs(XAP_DialogFactory * pDlgFactory,
 														   XAP_Dialog_Id id)
+#if GTK_CHECK_VERSION(2,4,0)
+  : XAP_Dialog_FileOpenSaveAs(pDlgFactory,id), m_FC(0), m_preview(0), m_bExport(true)
+#else
   : XAP_Dialog_FileOpenSaveAs(pDlgFactory,id), m_FS(0), m_preview(0), m_bExport(true)
+#endif
 {
 	m_szFinalPathnameCandidate = NULL;
 }
@@ -83,19 +90,42 @@ XAP_UnixDialog_FileOpenSaveAs::~XAP_UnixDialog_FileOpenSaveAs(void)
 
 /*****************************************************************/
 
-static void s_ok_clicked(GtkWidget * /* widget */,
-						 XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
+static void s_dialog_response(GtkWidget * /* widget */,
+						gint answer,
+						XAP_Dialog_FileOpenSaveAs::tAnswer * ptr)
 {
-	*answer = XAP_Dialog_FileOpenSaveAs::a_OK;
-	gtk_main_quit();
+	switch (answer)
+	{
+		case GTK_RESPONSE_CANCEL:
+		case GTK_RESPONSE_ACCEPT:
+
+			if (answer == GTK_RESPONSE_CANCEL)
+				*ptr = XAP_Dialog_FileOpenSaveAs::a_CANCEL;
+			else
+				*ptr = XAP_Dialog_FileOpenSaveAs::a_OK;
+			gtk_main_quit();
+			break;
+		default:
+			// do nothing
+			break;
+	}
+}
+
+#if !GTK_CHECK_VERSION(2,4,0)
+static void s_ok_clicked(GtkWidget * /* widget */,
+                                                 XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
+{
+        *answer = XAP_Dialog_FileOpenSaveAs::a_OK;
+        gtk_main_quit();
 }
 
 static void s_cancel_clicked(GtkWidget * /* widget */,
-							 XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
+                                                         XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
 {
-	*answer = XAP_Dialog_FileOpenSaveAs::a_CANCEL;
-	gtk_main_quit();
+        *answer = XAP_Dialog_FileOpenSaveAs::a_CANCEL;
+        gtk_main_quit();
 }
+#endif
 
 static void s_delete_clicked(GtkWidget * /* widget*/, gpointer /* data */, XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
 {
@@ -133,6 +163,11 @@ fsel_key_event (GtkWidget *widget, GdkEventKey *event, XAP_Dialog_FileOpenSaveAs
 	return FALSE;
 }
 
+static void s_file_activated(GtkWidget * w, XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
+{
+	s_dialog_response(w, GTK_RESPONSE_ACCEPT, answer);
+}
+
 static void file_selection_changed  (GtkTreeSelection  *selection,
 				     gpointer           ptr)
 {
@@ -143,12 +178,9 @@ static void file_selection_changed  (GtkTreeSelection  *selection,
 }
 
 bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
-													 void * pFSvoid,
 													 bool bCheckWritePermission,
 													 GtkWidget * filetypes_pulldown)
 {
-	GtkFileSelection * pFS = static_cast<GtkFileSelection *>(pFSvoid);
-
 	/*
 	  Run the dialog in a loop to catch bad filenames.
 	  The location of this check being in this dialog loop
@@ -195,7 +227,11 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			// a file, so we have to catch it, change the dialog, and not return
 			// any filename yet.
 
-			UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(pFS));
+#if GTK_CHECK_VERSION(2,4,0)
+			UT_cloneString(szDialogFilename, gtk_file_chooser_get_filename(m_FC));
+#else
+			UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(m_FS));
+#endif
 			UT_ASSERT(szDialogFilename);
 
 			err = stat(szDialogFilename, &buf);
@@ -210,7 +246,11 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 				{
 					g_string_append_c(s, '/');
 				}
-				gtk_file_selection_set_filename(pFS, s->str);
+#if GTK_CHECK_VERSION(2,4,0)				
+				gtk_file_chooser_set_filename(m_FC, s->str);
+#else
+				gtk_file_selection_set_filename(m_FS, s->str);
+#endif
 				g_string_free(s, TRUE);
 
 				// free the string and continue along
@@ -236,8 +276,13 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 
 		// Give us a filename we can mangle
 
-		UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(pFS));
-		UT_ASSERT(szDialogFilename);
+#if GTK_CHECK_VERSION(2,4,0)	
+		UT_cloneString(szDialogFilename, gtk_file_chooser_get_filename(m_FC));
+#else
+		UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(m_FS));
+#endif
+		if (!szDialogFilename)
+			continue;
 
 		// We append the suffix of the default type, so the user doesn't
 	        // have to.  This is adapted from the Windows front-end code
@@ -348,7 +393,11 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			{
 				g_string_append_c(s, '/');
 			}
-			gtk_file_selection_set_filename(pFS, s->str);
+#if GTK_CHECK_VERSION(2,4,0)			
+			gtk_file_chooser_set_filename(m_FC, s->str);
+#else
+			gtk_file_selection_set_filename(m_FS, s->str);
+#endif
 			g_string_free(s, TRUE);
 			goto ContinueLoop;
 		}
@@ -358,7 +407,11 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 		// for a matching directory.  We can then proceed with the file
 		// if another stat of that dir passes.
 
-		pLastSlash = strrchr(szFinalPathnameCopy,'/');
+		if (szFinalPathnameCopy && strlen(szFinalPathnameCopy))
+			pLastSlash = strrchr(szFinalPathnameCopy,'/');
+		else
+			pLastSlash = NULL;
+
 		if (!pLastSlash)
 		{
 			_notifyError_OKOnly(pFrame,XAP_STRING_ID_DLG_InvalidPathname);
@@ -456,10 +509,14 @@ void XAP_UnixDialog_FileOpenSaveAs::fileTypeChanged(GtkWidget * w)
 	{
 		return;
 	}
+#if GTK_CHECK_VERSION(2,4,0)			
+	UT_String sFileName = 	gtk_file_chooser_get_filename(m_FC);
+#else
 	UT_String sFileName = 	gtk_file_selection_get_filename(m_FS);
+#endif
 	UT_String sSuffix = m_szSuffixes[nFileType-1];
 	sSuffix = sSuffix.substr(1,sSuffix.length()-1);
-	UT_uint32 i = 0;
+	UT_sint32 i = 0;
 	bool bFoundComma = false;
 	for(i=0; i< sSuffix.length(); i++)
 	{
@@ -495,7 +552,15 @@ void XAP_UnixDialog_FileOpenSaveAs::fileTypeChanged(GtkWidget * w)
 	}
 	sFileName = sFileName.substr(0,i);
 	sFileName += sSuffix;
+	
+#if GTK_CHECK_VERSION(2,4,0)
+	if (!gtk_file_chooser_select_filename(m_FC,sFileName.c_str()))
+	{
+		gtk_file_chooser_set_current_name(m_FC, UT_basename(sFileName.c_str()));
+	}
+#else	
 	gtk_file_selection_set_filename(m_FS,sFileName.c_str());
+#endif
 }
 
 /*****************************************************************/
@@ -516,50 +581,50 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	{
 		case XAP_DIALOG_ID_INSERT_PICTURE:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_Title);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_Title, szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel, szFileTypeLabel);
 				bCheckWritePermission = false;    
 				break;
 			}
 		case XAP_DIALOG_ID_FILE_OPEN:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_OpenTitle);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_OpenTitle,szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel,szFileTypeLabel);
 				bCheckWritePermission = false;
 				break;
 			}
 		case XAP_DIALOG_ID_FILE_IMPORT:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_ImportTitle);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_ImportTitle,szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel,szFileTypeLabel);
 				bCheckWritePermission = false;
 				break;
 			}
 		case XAP_DIALOG_ID_INSERT_FILE:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_InsertTitle);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_InsertTitle,szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel,szFileTypeLabel);
 				bCheckWritePermission = false;
 				break;
 			}
 		case XAP_DIALOG_ID_FILE_SAVEAS:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_SaveAsTitle);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_SaveAsTitle,szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel,szFileTypeLabel);
 				bCheckWritePermission = true;
 				break;
 			}
 		case XAP_DIALOG_ID_FILE_EXPORT:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_ExportTitle);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_ExportTitle,szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel,szFileTypeLabel);
 				bCheckWritePermission = true;
 				break;
 			}
 		case XAP_DIALOG_ID_PRINTTOFILE:
 			{
-				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_PrintToFileTitle);
-				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FilePrintTypeLabel);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_PrintToFileTitle,szTitle);
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FilePrintTypeLabel,szFileTypeLabel);
 				bCheckWritePermission = true;
 				break;
 			}
@@ -573,36 +638,80 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	// NOTE: title and the error/confirmation message boxes.  we
 	// NOTE: let GTK take care of the localization of the actual
 	// NOTE: buttons and labels on the FileSelection dialog.
-	
+
+#if GTK_CHECK_VERSION(2,4,0)
+	// Get the GtkWindow of the parent frame
+	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(pFrame->getFrameImpl());
+	GtkWidget * parent = pUnixFrameImpl->getTopLevelWindow();
+
+	m_FC = GTK_FILE_CHOOSER( gtk_file_chooser_dialog_new (szTitle.utf8_str(),
+									GTK_WINDOW(parent),
+									(m_id == XAP_DIALOG_ID_FILE_OPEN || m_id == XAP_DIALOG_ID_INSERT_PICTURE || m_id == XAP_DIALOG_ID_FILE_EXPORT || m_id == XAP_DIALOG_ID_INSERT_FILE ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE),
+									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+									GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+									NULL)
+							);
+
+	abiSetupModalDialog(GTK_DIALOG(m_FC), pFrame, this, GTK_RESPONSE_ACCEPT);
+#else	
 	GtkFileSelection *pFS = GTK_FILE_SELECTION(gtk_file_selection_new(szTitle.utf8_str()));
 	m_FS = pFS;
 
 	abiSetupModalDialog(GTK_DIALOG(pFS), pFrame, this, GTK_RESPONSE_CANCEL);
+#endif
 	GtkWidget * filetypes_pulldown = NULL;
+
+	UT_UTF8String s;
 	
 	/*
-	  To facilitate a file-types selection, we dig around in some
-	  private data for the dialog layout, and add a drop-down list
+	  To facilitate a file-types selection, we dig around (only for GTK < 2.4)
+	  in some private data for the dialog layout, and add a drop-down list
 	  of known types.  We store an indexer in the user data
 	  for each menu item in the popup, so we can read the type
 	  we need to return.
 	*/
 	{
+#if !GTK_CHECK_VERSION(2,4,0)		
 		GtkWidget * main_vbox = pFS->main_vbox;
 		UT_ASSERT(main_vbox);
-
+#endif
+		
 		// hbox for our pulldown menu (GTK does its pulldown this way */
 		GtkWidget * pulldown_hbox = gtk_hbox_new(FALSE, 15);
+#if !GTK_CHECK_VERSION(2,4,0)		
 		gtk_box_pack_start(GTK_BOX(main_vbox), pulldown_hbox, TRUE, TRUE, 0);
+#endif
 		gtk_widget_show(pulldown_hbox);
+#if GTK_CHECK_VERSION(2,4,0)
+		gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER(m_FC), pulldown_hbox);
+#endif
 
 		if (m_id == XAP_DIALOG_ID_INSERT_PICTURE)
 		  {
 			  GtkWidget * preview = createDrawingArea ();
 		    gtk_widget_show (preview);
-		    m_preview = preview;
+			m_preview = preview;			  
+#if GTK_CHECK_VERSION(2,4,0)
+			gtk_widget_set_size_request (preview, PREVIEW_WIDTH, PREVIEW_HEIGHT);
 
-		    GtkWidget * frame = gtk_frame_new (pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_Activate_Label).utf8_str());
+			// place the preview area inside a container to get a nice border
+			GtkWidget * preview_hbox = gtk_hbox_new(FALSE, 0);
+			gtk_container_set_border_width  (GTK_CONTAINER(preview_hbox), 4);
+			gtk_box_pack_start(GTK_BOX(preview_hbox), preview, TRUE, TRUE, 0);
+
+			// attach the preview area to the dialog
+			gtk_file_chooser_set_preview_widget (m_FC, preview_hbox);
+			gtk_file_chooser_set_preview_widget_active (m_FC, true);
+
+			// connect some signals
+			g_signal_connect (m_FC, "update_preview",
+									G_CALLBACK (file_selection_changed), static_cast<gpointer>(this));
+
+			g_signal_connect (preview, "expose_event",
+									G_CALLBACK (s_preview_exposed), static_cast<gpointer>(this));
+#else
+			pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_Activate_Label,s);
+		    GtkWidget * frame = gtk_frame_new (s.utf8_str());
 			gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 		    gtk_widget_show (frame);
 		    gtk_container_add (GTK_CONTAINER(frame), preview);
@@ -617,7 +726,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 				       static_cast<gpointer>(this));
 
 		    g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (pFS->file_list)), "changed", G_CALLBACK (file_selection_changed), this);
-
+#endif
 		  }
 
 		// pulldown label
@@ -660,7 +769,8 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 			// Auto-detect is always an option, but a special one, so we use
 			// a pre-defined constant for the type, and don't use the user-supplied
 			// types yet.
-			g_snprintf(buffer, 1024, "%s", pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileTypeAutoDetect).utf8_str());
+			pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileTypeAutoDetect,s);
+			g_snprintf(buffer, 1024, "%s", s.utf8_str());
 			thismenuitem = gtk_menu_item_new_with_label(buffer);
 			g_object_set_data(G_OBJECT(thismenuitem), "user_data", GINT_TO_POINTER(XAP_DIALOG_FILEOPENSAVEAS_FILE_TYPE_AUTO));
 			gtk_widget_show(thismenuitem);
@@ -714,20 +824,49 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	}
 	
 	// connect the signals for OK and CANCEL and the requisite clean-close signals
-	g_signal_connect(G_OBJECT(pFS),
+#if GTK_CHECK_VERSION(2,4,0)
+	g_signal_connect(G_OBJECT(m_FC),
 							 "delete_event",
 							 G_CALLBACK(s_delete_clicked),
 							 &m_answer);
+#else
+	g_signal_connect(G_OBJECT(m_FS),
+							 "delete_event",
+							 G_CALLBACK(s_delete_clicked),
+							 &m_answer);
+#endif
 
-	g_signal_connect (G_OBJECT (pFS),
+#if GTK_CHECK_VERSION(2,4,0)
+	g_signal_connect(G_OBJECT(m_FC),
 			    "key_press_event",
 			    G_CALLBACK(fsel_key_event), &m_answer);
-	g_signal_connect(G_OBJECT(pFS->ok_button), "clicked",
-					   G_CALLBACK(s_ok_clicked), &m_answer);
-	g_signal_connect(G_OBJECT(pFS->cancel_button), "clicked",
-					   G_CALLBACK(s_cancel_clicked), &m_answer);
+#else
+	g_signal_connect(G_OBJECT(m_FS),
+			    "key_press_event",
+			    G_CALLBACK(fsel_key_event), &m_answer);
+#endif
+
+#if GTK_CHECK_VERSION(2,4,0)
+	g_signal_connect (G_OBJECT (m_FC),
+				"response",
+				G_CALLBACK(s_dialog_response), &m_answer);
+#else
+        g_signal_connect(G_OBJECT(pFS->ok_button), "clicked",
+                                           G_CALLBACK(s_ok_clicked), &m_answer);
+        g_signal_connect(G_OBJECT(pFS->cancel_button), "clicked",
+                                           G_CALLBACK(s_cancel_clicked), &m_answer);
+#endif
+	
+#if GTK_CHECK_VERSION(2,4,0)
+	g_signal_connect (G_OBJECT (m_FC),
+				"file-activated",
+				G_CALLBACK(s_file_activated), &m_answer);	
+#endif
+
+#if !GTK_CHECK_VERSION(2,4,0)
 	if (m_id == XAP_DIALOG_ID_FILE_OPEN || m_id == XAP_DIALOG_ID_INSERT_PICTURE || m_id == XAP_DIALOG_ID_FILE_EXPORT || m_id == XAP_DIALOG_ID_INSERT_FILE) // only hide the buttons if we're opening a file/picture
-	  gtk_file_selection_hide_fileop_buttons(pFS);
+	  gtk_file_selection_hide_fileop_buttons(m_FS);
+#endif
 
 	// use the persistence info and/or the suggested filename
 	// to properly seed the dialog.
@@ -751,7 +890,11 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 			char * pLastSlash = strrchr(szPersistDirectory, '/');
 			if (pLastSlash)
 				pLastSlash[1] = 0;
-			gtk_file_selection_set_filename(pFS,szPersistDirectory);
+#if GTK_CHECK_VERSION(2,4,0)
+			gtk_file_chooser_set_filename(m_FC,szPersistDirectory);
+#else
+			gtk_file_selection_set_filename(m_FS,szPersistDirectory);
+#endif
 		}
 		else
 		{
@@ -771,8 +914,11 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		if (m_bSuggestName)
 		{
 			// use m_szInitialPathname
-			
-			gtk_file_selection_set_filename(pFS, m_szInitialPathname);
+#if GTK_CHECK_VERSION(2,4,0)			
+			gtk_file_chooser_set_filename(m_FC, m_szInitialPathname);
+#else
+			gtk_file_selection_set_filename(m_FS, m_szInitialPathname);
+#endif
 		}
 		else
 		{
@@ -782,10 +928,21 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 			char * pLastSlash = strrchr(szPersistDirectory, '/');
 			if (pLastSlash)
 				pLastSlash[1] = 0;
-			gtk_file_selection_set_filename(pFS,szPersistDirectory);
+#if GTK_CHECK_VERSION(2,4,0)	
+			gtk_file_chooser_set_filename(m_FC,szPersistDirectory);
+#else
+			gtk_file_selection_set_filename(m_FS,szPersistDirectory);
+#endif
 		}
 	}
 
+#if GTK_CHECK_VERSION(2,4,0)
+	// center the dialog
+	centerDialog(parent, GTK_WIDGET(m_FC));
+
+	gtk_widget_show(GTK_WIDGET(m_FC));
+	gtk_grab_add(GTK_WIDGET(m_FC));
+#else
 	// get top level window and its GtkWidget *
 	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(pFrame->getFrameImpl());
 	UT_ASSERT(pUnixFrameImpl);
@@ -797,8 +954,9 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	
 	gtk_widget_show(GTK_WIDGET(pFS));
 	gtk_grab_add(GTK_WIDGET(pFS));
-
-	bool bResult = _run_gtk_main(pFrame,pFS,bCheckWritePermission,filetypes_pulldown);
+#endif
+	
+	bool bResult = _run_gtk_main(pFrame,bCheckWritePermission,filetypes_pulldown);
 	
 	if (bResult)
 	{
@@ -814,9 +972,14 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		UT_ASSERT(activeItem);
 		m_nFileType = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(activeItem), "user_data"));
 	}
-			  
-	if(pFS && GTK_IS_WIDGET(pFS))
-	  gtk_widget_destroy (GTK_WIDGET(pFS));
+
+#if GTK_CHECK_VERSION(2,4,0)
+	if(m_FC && GTK_IS_WIDGET(m_FC))
+	  gtk_widget_destroy (GTK_WIDGET(m_FC));
+#else	
+	if(m_FS && GTK_IS_WIDGET(m_FS))
+	  gtk_widget_destroy (GTK_WIDGET(m_FS));
+#endif
 
 	FREEP(szPersistDirectory);
 
@@ -825,7 +988,11 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 
 gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 {
-        UT_ASSERT (m_FS && m_preview);
+#if GTK_CHECK_VERSION(2,4,0)	
+	UT_ASSERT (m_FC && m_preview);
+#else
+	UT_ASSERT (m_FS && m_preview);
+#endif
 
 	XAP_UnixApp * unixapp = static_cast<XAP_UnixApp *> (m_pApp);
 	UT_ASSERT(unixapp);
@@ -833,20 +1000,21 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
 	// attach and clear the area immediately
-#ifndef WITH_PANGO	
-	GR_UnixGraphics* pGr = new GR_UnixGraphics(m_preview->window, unixapp->getFontManager(), m_pApp);
+	//GR_UnixGraphics* pGr = new GR_UnixGraphics(m_preview->window, unixapp->getFontManager(), m_pApp);
+	GR_UnixAllocInfo ai(m_preview->window, unixapp->getFontManager(), m_pApp);
+	GR_UnixGraphics* pGr = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
+
+#if GTK_CHECK_VERSION(2,4,0)	
+	const gchar * file_name = gtk_file_chooser_get_filename (m_FC);
 #else
-	GR_UnixGraphics* pGr = new GR_UnixGraphics(m_preview->window, m_pApp);
+	const gchar * file_name = gtk_file_selection_get_filename (m_FS);
 #endif
 	
-	pGr->clearArea(0, 0, pGr->tlu(m_preview->allocation.width), pGr->tlu(m_preview->allocation.height));
-
-	const gchar * buf = gtk_file_selection_get_filename (m_FS);
-
 	GR_Font * fnt = pGr->findFont("Times New Roman", "normal", "", "normal", "", "12pt");
 	pGr->setFont(fnt);
 
-	UT_UTF8String str(pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_No_Picture_Label).utf8_str());
+	UT_UTF8String str;
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_No_Picture_Label, str);
 
 	int answer = 0;
 
@@ -860,51 +1028,39 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	UT_sint32     scaled_width,scaled_height;
 	UT_sint32     iImageWidth,iImageHeight;
 
-	if (!buf)
+	{
+	GR_Painter painter(pGr);
+	painter.clearArea(0, 0, pGr->tlu(m_preview->allocation.width), pGr->tlu(m_preview->allocation.height));
+
+	if (!file_name)
 	  {
-#ifndef WITH_PANGO 		  
-	    pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
-#else
-		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
-#endif		
+		painter.drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(static_cast<int>(m_preview->allocation.height / 2)) - pGr->getFontHeight(fnt)/2);
 	    goto Cleanup;
 	  }
 
 	// are we dealing with a file or directory here?
 	struct stat st;
-	if (!stat (buf, &st)) {
+	if (!stat (file_name, &st)) {
 		if (!S_ISREG(st.st_mode)) {
-#ifndef WITH_PANGO 		  
-			pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
-#else
-			pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
-#endif		
+			painter.drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(static_cast<int>(m_preview->allocation.height / 2)) - pGr->getFontHeight(fnt)/2);
 			goto Cleanup;
 		}
 	}
 	else {
-#ifndef WITH_PANGO 		  
-		pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
-#else
-		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
-#endif		
+		painter.drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(static_cast<int>(m_preview->allocation.height / 2)) - pGr->getFontHeight(fnt)/2);
 		goto Cleanup;
 	}
 
 	// Load File into memory
 	pBB     = new UT_ByteBuf(0);
-	pBB->insertFromFile(0, buf);
+	pBB->insertFromFile(0, file_name);
 
 	// Build an Import Graphic based on file type
-	errorCode = IE_ImpGraphic::constructImporter(buf, IEGFT_Unknown, &pIEG);
+	errorCode = IE_ImpGraphic::constructImporter(file_name, IEGFT_Unknown, &pIEG);
 	if ((errorCode != UT_OK) || !pIEG)
 	{
 		DELETEP(pBB);
-#ifndef WITH_PANGO 		  
-		pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
-#else
-		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
-#endif		
+		painter.drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(static_cast<int>(m_preview->allocation.height / 2)) - pGr->getFontHeight(fnt)/2);
 		goto Cleanup;
 	}
 
@@ -912,11 +1068,7 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 
 	if ((errorCode != UT_OK) || !pGraphic)
 	  {
-#ifndef WITH_PANGO 		  
-		pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
-#else
-		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
-#endif		
+		painter.drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(static_cast<int>(m_preview->allocation.height / 2)) - pGr->getFontHeight(fnt)/2);
 	    goto Cleanup;
 	  }
 
@@ -938,7 +1090,7 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 
 		pImage->convertFromBuffer(png, scaled_width, scaled_height);
 		
-		pGr->drawImage(pImage,
+		painter.drawImage(pImage,
 			       pGr->tlu(static_cast<int>((m_preview->allocation.width  - scaled_width ) / 2)),
 			       pGr->tlu(static_cast<int>((m_preview->allocation.height - scaled_height) / 2)));
 		
@@ -948,11 +1100,11 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	{
 	  UT_ASSERT_NOT_REACHED ();
 	}
-
+	}
+	
  Cleanup:
 	DELETEP(pImage);
 	DELETEP(pGr);
-	DELETEP(fnt);
 	DELETEP(pIEG);
 	DELETEP(pGraphic);
 
