@@ -63,7 +63,7 @@ void AP_Win32Dialog_Replace::activate(void)
 
 	// Update the caption
 	ConstructWindowName();
-	SetWindowText(m_hWnd, (AP_Win32App::s_fromUTF8ToAnsi(m_WindowName)).c_str()); 
+	SetWindowText(m_hWnd, (AP_Win32App::s_fromUTF8ToWinLocale(m_WindowName)).c_str()); 
 
 	SetFocus( GetDlgItem( m_hWnd,AP_RID_DIALOG_REPLACE_COMBO_FIND) );
 
@@ -71,7 +71,7 @@ void AP_Win32Dialog_Replace::activate(void)
 
 	iResult = BringWindowToTop( m_hWnd );
 	
-	UT_ASSERT((iResult != 0));
+	UT_return_if_fail ((iResult != 0));
 	
 	{
 	UT_UCSChar * bufferUnicode = getFindString();
@@ -107,7 +107,7 @@ void AP_Win32Dialog_Replace::destroy(void)
 {
 	int iResult = DestroyWindow( m_hWnd );
 
-	UT_ASSERT((iResult != 0));
+	UT_ASSERT_HARMLESS((iResult != 0));
 
 	modeless_cleanup();
 }
@@ -118,7 +118,7 @@ void AP_Win32Dialog_Replace::notifyActiveFrame(XAP_Frame *pFrame)
 	{
 		// Update the caption
 		ConstructWindowName();
-		SetWindowText(m_hWnd, (AP_Win32App::s_fromUTF8ToAnsi(m_WindowName)).c_str()); 
+		SetWindowText(m_hWnd, (AP_Win32App::s_fromUTF8ToWinLocale(m_WindowName)).c_str()); 
 
 		SetWindowLong(m_hWnd, GWL_HWNDPARENT, (long)static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow());
 		SetWindowPos(m_hWnd, NULL, 0, 0, 0, 0,
@@ -147,7 +147,7 @@ void AP_Win32Dialog_Replace::runModeless(XAP_Frame * pFrame)
 		lpTemplate = MAKEINTRESOURCE(AP_RID_DIALOG_FIND);
 	else
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
@@ -158,7 +158,7 @@ void AP_Win32Dialog_Replace::runModeless(XAP_Frame * pFrame)
 							static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow(),
 							(DLGPROC)s_dlgProc,(LPARAM)this);
 
-	UT_ASSERT((hResult != NULL));
+	UT_return_if_fail ((hResult != NULL));
 
 	m_hWnd = hResult;
 
@@ -168,12 +168,12 @@ void AP_Win32Dialog_Replace::runModeless(XAP_Frame * pFrame)
 
 	// Update the caption
 	ConstructWindowName();
-	SetWindowText(m_hWnd, (AP_Win32App::s_fromUTF8ToAnsi(m_WindowName)).c_str()); 
+	SetWindowText(m_hWnd, (AP_Win32App::s_fromUTF8ToWinLocale(m_WindowName)).c_str()); 
 
 	iResult = ShowWindow( m_hWnd, SW_SHOW );
 
 	iResult = BringWindowToTop( m_hWnd );
-	UT_ASSERT((iResult != 0));
+	UT_ASSERT_HARMLESS((iResult != 0));
 	m_pView->focusChange(AV_FOCUS_MODELESS);
 }
 
@@ -315,10 +315,13 @@ BOOL AP_Win32Dialog_Replace::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		if (!m_pView->isSelectionEmpty()) 
 		{
 		// if there's a selection, clear it
-			if (m_pView->getSelectionText() != NULL) 
+			UT_UCS4Char * pSelection;
+			m_pView->getSelectionText(pSelection);
+			
+			if ( pSelection != NULL) 
 			{
 				PT_DocPosition pt = m_pView->getSelectionAnchor();
-				PT_DocPosition ln = UT_UCS4_strlen (m_pView->getSelectionText());
+				PT_DocPosition ln = UT_UCS4_strlen (pSelection);
 				if (currentVal)
 				{
 					m_pView->moveInsPtTo(pt);
@@ -329,6 +332,8 @@ BOOL AP_Win32Dialog_Replace::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 				}
 				m_pView->cmdUnselectSelection();
 			}
+
+			FREEP(pSelection);
 		}
 
 		m_pView->findSetStartAtInsPoint();
@@ -336,13 +341,13 @@ BOOL AP_Win32Dialog_Replace::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		return 1;	
 		}
 	case AP_RID_DIALOG_REPLACE_BTN_FINDNEXT:
-		return _onBtn_FindNext(hWnd);
+		return _onBtn_Find(hWnd, find_FIND_NEXT);
 		
 	case AP_RID_DIALOG_REPLACE_BTN_REPLACE:
-		return _onBtn_Replace(hWnd);
+		return _onBtn_Find(hWnd, find_REPLACE);
 		
 	case AP_RID_DIALOG_REPLACE_BTN_REPLACEALL:
-		return _onBtn_ReplaceAll(hWnd);
+		return _onBtn_Find(hWnd, find_REPLACE_ALL);
 		
 	case IDCANCEL:						// also AP_RID_DIALOG_REPLACE_BTN_CLOSE
 		EndDialog(hWnd,0);
@@ -354,56 +359,16 @@ BOOL AP_Win32Dialog_Replace::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	}
 }
 
-BOOL AP_Win32Dialog_Replace::_onBtn_FindNext(HWND hWnd)
+BOOL AP_Win32Dialog_Replace::_onBtn_Find(HWND hWnd, tFindType tFindType)
 {
 	char * pBufFromDialogFind = NULL;
 	UT_UCSChar * pUCSFind = NULL;
-
-	HWND hWndEditFind = GetDlgItem(hWnd,AP_RID_DIALOG_REPLACE_COMBO_FIND);
-	DWORD lenFind = GetWindowTextLength(hWndEditFind);
-	if (!lenFind)
-		return 1;
-
-	pBufFromDialogFind = new char [lenFind + 1];
-	if (!pBufFromDialogFind)
-		goto FreeMemory;
-	GetWindowText(hWndEditFind,pBufFromDialogFind,lenFind+1);
-
-	UT_DEBUGMSG(("Find entry contents: [%s]\n",pBufFromDialogFind));
-
-	UT_UCS4_cloneString_char(&pUCSFind,pBufFromDialogFind);
-	if (!pUCSFind)
-		goto FreeMemory;
-
-	setFindString(pUCSFind);
-	if (!getReverseFind()) {
-    	findNext();
-		}
-	else {
-		findPrev();
-		}
-
-FreeMemory:
-	DELETEP(pBufFromDialogFind);
-	FREEP(pUCSFind);
-
-	return 1;
-}
-
-BOOL AP_Win32Dialog_Replace::_onBtn_Replace(HWND hWnd)
-{
-	UT_ASSERT((m_id == AP_DIALOG_ID_REPLACE));
-
-	char * pBufFromDialogFind = NULL;
 	char * pBufFromDialogReplace = NULL;
-	UT_UCSChar * pUCSFind = NULL;
 	UT_UCSChar * pUCSReplace = NULL;
 
+	// Check find string
 	HWND hWndEditFind = GetDlgItem(hWnd,AP_RID_DIALOG_REPLACE_COMBO_FIND);
-	HWND hWndEditReplace = GetDlgItem(hWnd,AP_RID_DIALOG_REPLACE_COMBO_REPLACE);
 	DWORD lenFind = GetWindowTextLength(hWndEditFind);
-	DWORD lenReplace = GetWindowTextLength(hWndEditReplace);
-
 	if (!lenFind)
 		return 1;
 
@@ -418,84 +383,54 @@ BOOL AP_Win32Dialog_Replace::_onBtn_Replace(HWND hWnd)
 	if (!pUCSFind)
 		goto FreeMemory;
 
-	pBufFromDialogReplace = new char [lenReplace + 1];
-	if (!pBufFromDialogReplace)
-		goto FreeMemory;
-	GetWindowText(hWndEditReplace,pBufFromDialogReplace,lenReplace+1);
-
-	UT_DEBUGMSG(("Replace entry contents: [%s]\n",pBufFromDialogReplace));
-
-	UT_UCS4_cloneString_char(&pUCSReplace,pBufFromDialogReplace);
-	if (!pUCSReplace)
-		goto FreeMemory;
-
 	setFindString(pUCSFind);
-	setReplaceString(pUCSReplace);
-	if (!getReverseFind()) {
-    	findReplace();
-		}
-	else {
-		findReplaceReverse();
-		}
+
+	if (m_id == AP_DIALOG_ID_REPLACE)
+	{
+		// Check Replace string
+		HWND hWndEditReplace = GetDlgItem(hWnd,AP_RID_DIALOG_REPLACE_COMBO_REPLACE);
+		DWORD lenReplace = GetWindowTextLength(hWndEditReplace);
+
+		pBufFromDialogReplace = new char [lenReplace + 1];
+		if (!pBufFromDialogReplace)
+			goto FreeMemory;
+		GetWindowText(hWndEditReplace,pBufFromDialogReplace,lenReplace+1);
+
+		UT_DEBUGMSG(("Replace entry contents: [%s]\n",pBufFromDialogReplace));
+
+		UT_UCS4_cloneString_char(&pUCSReplace,pBufFromDialogReplace);
+		if (!pUCSReplace)
+			goto FreeMemory;
+
+		setReplaceString(pUCSReplace);
+	}
+
+	if (tFindType == find_REPLACE_ALL)
+	{
+		UT_return_val_if_fail ((m_id == AP_DIALOG_ID_REPLACE),0); //should never happen in Find dialog
+		findReplaceAll();
+	}
+	else if (tFindType == find_REPLACE)
+	{
+		UT_return_val_if_fail ((m_id == AP_DIALOG_ID_REPLACE),0); //should never happen in Find dialog
+		if (!getReverseFind())
+			findReplace();
+		else
+			findReplaceReverse();
+	}
+	else if (tFindType == find_FIND_NEXT)
+	{
+		if (!getReverseFind())
+			findNext();
+		else
+			findPrev();
+	}
 
 FreeMemory:
 	DELETEP(pBufFromDialogFind);
+	FREEP(pUCSFind);
 	DELETEP(pBufFromDialogReplace);
 	FREEP(pUCSFind);
-	FREEP(pUCSReplace);
-	
-	return 1;
-}
-
-BOOL AP_Win32Dialog_Replace::_onBtn_ReplaceAll(HWND hWnd)
-{
-	UT_ASSERT((m_id == AP_DIALOG_ID_REPLACE));
-
-	char * pBufFromDialogFind = NULL;
-	char * pBufFromDialogReplace = NULL;
-	UT_UCSChar * pUCSFind = NULL;
-	UT_UCSChar * pUCSReplace = NULL;
-
-	HWND hWndEditFind = GetDlgItem(hWnd,AP_RID_DIALOG_REPLACE_COMBO_FIND);
-	HWND hWndEditReplace = GetDlgItem(hWnd,AP_RID_DIALOG_REPLACE_COMBO_REPLACE);
-	DWORD lenFind = GetWindowTextLength(hWndEditFind);
-	DWORD lenReplace = GetWindowTextLength(hWndEditReplace);
-
-	if (!lenFind)
-		return 1;
-
-	pBufFromDialogFind = new char [lenFind + 1];
-	if (!pBufFromDialogFind)
-		goto FreeMemory;
-	GetWindowText(hWndEditFind,pBufFromDialogFind,lenFind+1);
-
-	UT_DEBUGMSG(("Find entry contents: [%s]\n",pBufFromDialogFind));
-
-	UT_UCS4_cloneString_char(&pUCSFind,pBufFromDialogFind);
-	if (!pUCSFind)
-		goto FreeMemory;
-
-	pBufFromDialogReplace = new char [lenReplace + 1];
-	if (!pBufFromDialogReplace)
-		goto FreeMemory;
-	GetWindowText(hWndEditReplace,pBufFromDialogReplace,lenReplace+1);
-
-	UT_DEBUGMSG(("Replace entry contents: [%s]\n",pBufFromDialogReplace));
-
-	UT_UCS4_cloneString_char(&pUCSReplace,pBufFromDialogReplace);
-	if (!pUCSReplace)
-		goto FreeMemory;
-
-	setFindString(pUCSFind);
-	setReplaceString(pUCSReplace);
-	findReplaceAll();
-
-FreeMemory:
-	DELETEP(pBufFromDialogFind);
-	DELETEP(pBufFromDialogReplace);
-	FREEP(pUCSFind);
-	FREEP(pUCSReplace);
-	
 	return 1;
 }
 
@@ -505,7 +440,7 @@ void AP_Win32Dialog_Replace::_updateLists()
 	_updateList(GetDlgItem(m_hWnd, AP_RID_DIALOG_REPLACE_COMBO_REPLACE), 	&m_replaceList);	
 }
 
-void AP_Win32Dialog_Replace::_updateList(HWND hWnd, UT_Vector* list)
+void AP_Win32Dialog_Replace::_updateList(HWND hWnd, UT_GenericVector<UT_UCS4Char*> *list)
 {
 	UT_DEBUGMSG(("AP_Win32Dialog_Replace::_updateList\n"));
 	
@@ -525,7 +460,7 @@ void AP_Win32Dialog_Replace::_updateList(HWND hWnd, UT_Vector* list)
 		// add it to the list
 		UT_DEBUGMSG(("FODDEX: find/replace list: %d = '%s'\n", i, utf8s));   
 				
-    	SendMessage(hWnd, CB_ADDSTRING, 0, (LPARAM)(AP_Win32App::s_fromUTF8ToAnsi(utf8s)).c_str());    		
+    	SendMessage(hWnd, CB_ADDSTRING, 0, (LPARAM)(AP_Win32App::s_fromUTF8ToWinLocale(utf8s)).c_str());    		
 
 		free(utf8s);
 	}		
