@@ -61,6 +61,11 @@
 #define _CV		EV_EMC_REVISION
 #define _CTV		EV_EMC_VLINE
 #define _CTH		EV_EMC_HLINE
+#define _CTF		EV_EMC_FRAME
+#define _CVD		EV_EMC_VISUALTEXTDRAG
+#define _CTC		EV_EMC_TOPCELL
+#define _CTO		EV_EMC_TOC
+#define _CPO		EV_EMC_POSOBJECT
 
 #define _B0		| EV_EMB_BUTTON0
 #define _B1		| EV_EMB_BUTTON1
@@ -92,6 +97,22 @@ ap_bs_Mouse MouseTable[] =
 	{_CH _B0,	{ "",	"",			"hyperlinkStatusBar",	"",			"",			"",				}},
 	{_CTV _B0,	{ "",	"",			"cursorVline",	"",			"",			"",				}},
 	{_CTH _B0,	{ "",	"",			"cursorHline",	"",			"",			"",				}},
+	{_CTF _B0,	{ "",	"",			"btn0Frame",	"",			"",			"",				}},
+	{_CPO _B0,	{ "",	"",			"btn0Frame",	"",			"",			"",				}},
+	{_CVD _B0,	{ "",	"",			"btn0VisualText",	"",			"",			"",				}},
+	{_CTC _B0,	{ "",	"",			"cursorTopCell",	"",			"",			"",				}},
+	{_CTO _B0,	{ "",	"",			"cursorTOC",	"",			"",			"",				}},
+
+
+//	Button-1, VisualTextDrag context
+//  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease }},
+  { _CVD _B1,	{ "cutVisualText",  "copyVisualText",	 "dragVisualText", "dragVisualText",   "pasteVisualText",	   	"pasteVisualText" }},
+  { _CVD _B1 _C,  { "copyVisualText",  "cutVisualText",	 "dragVisualText", "dragVisualText",   "pasteVisualText",	   	"pasteVisualText" }},
+
+//	Button-1, Frame-context
+//  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease }},
+  { _CTF _B1,	{ "btn1Frame",     "btn1Frame",		"dragFrame", "dragFrame",   "releaseFrame",	   	"releaseFrame" }},
+  { _CPO _B1,	{ "btn1Frame",     "btn1Frame",		"dragFrame", "dragFrame",   "releaseFrame",	   	"releaseFrame" }},
 
 
 //	Button-1, Image-context
@@ -107,6 +128,8 @@ ap_bs_Mouse MouseTable[] =
 	{_CT _B1 _S,{ "extSelToXY",		"",				"dragToXY", "",				"endDrag",	"endDrag"		}},
 	{_CT _B1 _C,{ "",		"",		       	"dragSelectionBegin",	       	"dragSelectionBegin",		        "dragSelectionEnd",	"dragSelectionEnd"		}},
 
+// Button-1 TOC-context
+	{_CTO _B1,	{ "selectTOC", "warpInsPtToXY",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
 
 //	Button-1, Left-of-Text-context (left-margin)
 //  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
@@ -132,6 +155,8 @@ ap_bs_Mouse MouseTable[] =
 
 	{_CTV _B1,	{ "beginVDrag",		 "clearSetCols", "dragVline",	"",		"endDragVline",	""				}},
 	{_CTH _B1,	{ "beginHDrag",		 "clearSetRows", "dragHline",	"",		"endDragHline",	""				}},
+	{_CTC _B1,	{ "selectColumnClick","clearSetCols", "dragHline",	"",		"endDragVline",	""				}},
+	{_CTC _B2,	{ "selectColumnClick","clearSetCols", "dragHline",	"",		"endDragVline",	""				}},
 
 
 //	Button-1, ImageSize-context
@@ -155,9 +180,12 @@ ap_bs_Mouse MouseTable[] =
 //  { context	{ click					dblclick	drag,	dbldrag,	release,	doublerelease	}},
 	//{_CU _B3,	{ "contextDefault",		"",			"",		"",			"",			""				}},
 	{_CT _B3,	{ "contextText",		"",			"",		"",			"",			""				}},
+	{_CVD _B3,  { "contextText",		"",			"",		"",			"",			""				}},
 	//{_CL _B3,	{ "contextLeftOfText",	"",			"",		"",			"",			""				}},
 	{_CM _B3,	{ "contextMisspellText","",			"",		"",			"",			""				}},
 	{_CI _B3,	{ "contextImage",		"",			"",		"",			"",			""				}},
+	{_CTF _B3,	{ "contextFrame",		"",			"",		"",			"",			""				}},
+	{_CPO _B3,	{ "contextFrame",		"",			"",		"",			"",			""				}},
 	//{_CZ _B3,	{ "contextImageSize",	"",			"",		"",			"",			""				}},
 	//{_CF _B3,	{ "contextField",		"",			"",		"",			"",			""				}},
 
@@ -190,6 +218,10 @@ ap_bs_Mouse MouseTable[] =
 	{_CTV _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CTH _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
 	{_CTH _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
+	{_CTO _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CTO _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
+	{_CVD _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CVD _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 
 
 //  Button-5, Mouse wheel (3 lines down)
@@ -217,6 +249,10 @@ ap_bs_Mouse MouseTable[] =
 	{_CTV _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CTH _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
 	{_CTH _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
+	{_CVD _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CVD _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
+	{_CTO _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CTO _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 
 #if defined(PT_TEST) || defined(FMT_TEST) || defined(UT_TEST)
 //  { context		{ click			doubleclick		drag,		dbldrag,	release,	doublerelease	}},
@@ -249,7 +285,7 @@ ap_bs_NVK NVKTable[] =
 						  "",					"",					"",				""					}},
 	{EV_NVK_SPACE,		{ "insertSpace",		"insertSpace",		"togglePlain",	"insertNBSpace",
 						  "insertNBZWSpace",	"insertZWJoiner",					"",				""					}},
-	{EV_NVK_TAB,		{ "insertTab",			"",					"cycleWindows",	"cycleWindowsBck",
+	{EV_NVK_TAB,		{ "insertTab",			"insertTabShift",					"cycleWindows",	"cycleWindowsBck",
 						  DO_NOT_USE,			"",					"",				""					}},
 	{EV_NVK_RETURN,		{ "insertParagraphBreak", "insertLineBreak", "insertPageBreak", "insertColumnBreak",
 						  "insertSectionBreak",	"",					"",				""					}},
@@ -381,9 +417,9 @@ ap_bs_Char CharTable[] =
 	{0x28, /* (      */ { "insertOpeningParenthesis","",			"",				""					}},
 	{0x29, /* )      */ { "insertClosingParenthesis","",			"",				""					}},
 	{0x2a, /* *      */ { "insertData",			"viewPara",			"",				""					}},
-	{0x2b, /* +      */ { "insertData",			"",					"",				""					}},
+	{0x2b, /* +      */ { "insertData",			"zoomIn",			"",				""					}},
 	{0x2c, /* ,      */ { "insertData",			"",					"",				""					}},
-	{0x2d, /* -      */ { "insertData",			"toggleSub",		"zoomOut",		""					}},
+	{0x2d, /* -      */ { "insertData",			"zoomOut",			"",				""					}},
 	{0x2e, /* .      */ { "insertData",			"pasteSpecial",		"",				""					}},
 	{0x2f, /* /      */ { "insertData",			"",					"",				""					}},
 	{0x30, /* 0      */ { "insertData",			"",					DO_NOT_USE,		""					}},
@@ -399,7 +435,7 @@ ap_bs_Char CharTable[] =
 	{0x3a, /* :      */ { "insertData",			"",					"",				""					}},
 	{0x3b, /* ;      */ { "insertData",			"",					"",				""					}},
 	{0x3c, /* <      */ { "insertData",			"fontSizeIncrease",	"",				"insertRLM"			}},
-	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"zoomIn",		""					}},
+	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"",				""					}},
 	{0x3e, /* >      */ { "insertData",			"fontSizeDecrease",	"",				"insertLRM"			}},
 	{0x3f, /* ?      */ { "insertData",			"",					"",				""					}},
 	{0x40, /* @      */ { "insertData",			"",					"",				""					}},
@@ -416,12 +452,12 @@ ap_bs_Char CharTable[] =
 	{0x4b, /* K      */ { "insertData",			"toggleStrike",		DO_NOT_USE,		""					}},
 	{0x4c, /* L      */ { "insertData",			"alignLeft",		DO_NOT_USE,		""					}},
 	{0x4d, /* M      */ { "insertData",			"insSymbol",		DO_NOT_USE,		""					}},
-	{0x4e, /* N      */ { "insertData",			"toolbarNew",		DO_NOT_USE,		""					}},
+	{0x4e, /* N      */ { "insertData",			"fileNewUsingTemplate",	DO_NOT_USE,		""					}},
 	{0x4f, /* O      */ { "insertData",			"fileOpen",			DO_NOT_USE,		""					}},
 	{0x50, /* P      */ { "insertData",			"print",			DO_NOT_USE,		""					}},
 	{0x51, /* Q      */ { "insertData",			"querySaveAndExit",	DO_NOT_USE,		""					}},
 	{0x52, /* R      */ { "insertData",			"alignRight",		DO_NOT_USE,		""					}},
-	{0x53, /* S      */ { "insertData",			"fileSave",			DO_NOT_USE,		""					}},
+	{0x53, /* S      */ { "insertData",			"fileSaveAs", 		DO_NOT_USE,		""					}},
 	{0x54, /* T      */ { "insertData",			"toggleOline",		DO_NOT_USE,		""					}},
 	{0x55, /* U      */ { "insertData",			"toggleUline",		DO_NOT_USE,		""					}},
 	{0x56, /* V      */ { "insertData",			"paste",			DO_NOT_USE,		""					}},
@@ -448,12 +484,12 @@ ap_bs_Char CharTable[] =
 	{0x6b, /* k      */ { "insertData",			"toggleStrike",		DO_NOT_USE,		""					}},
 	{0x6c, /* l      */ { "insertData",			"alignLeft",		DO_NOT_USE,		""					}},
 	{0x6d, /* m      */ { "insertData",			"insSymbol",		DO_NOT_USE,		""					}},
-	{0x6e, /* n      */ { "insertData",			"toolbarNew",		DO_NOT_USE,		""					}},
+	{0x6e, /* n      */ { "insertData",			"fileNew",			DO_NOT_USE,		""					}},
 	{0x6f, /* o      */ { "insertData",			"fileOpen",			DO_NOT_USE,		""					}},
 	{0x70, /* p      */ { "insertData",			"print",			DO_NOT_USE,		""					}},
 	{0x71, /* q      */ { "insertData",			"querySaveAndExit",	DO_NOT_USE,		""					}},
 	{0x72, /* r      */ { "insertData",			"alignRight",		DO_NOT_USE,		""					}},
-	{0x73, /* s      */ { "insertData",			"fileSave",			DO_NOT_USE,		""					}},
+	{0x73, /* s      */ { "insertData",			"fileSave",		    DO_NOT_USE,		""					}},
 	{0x74, /* t      */ { "insertData",			"toggleOline",		DO_NOT_USE,		""					}},
 	{0x75, /* u      */ { "insertData",			"toggleUline",		DO_NOT_USE,		""					}},
 	{0x76, /* v      */ { "insertData",			"paste",			DO_NOT_USE,		""					}},
