@@ -154,8 +154,12 @@ struct XAP_Exp_HTMLOptions
 	bool	bDeclareXML;
 	bool	bAllowAWML;
 	bool	bEmbedCSS;
+	bool	bLinkCSS;
 	bool	bEmbedImages;
-
+	bool    bClassOnly;
+	bool    bAbsUnits;
+	bool	bScaleUnits;
+	UT_uint32 iCompact;
 	/* other options, not set/saved/restore by options dialog
 	 */
 	bool	bMultipart;
@@ -184,10 +188,7 @@ public:
 	inline void			set_HTML4 (bool enable = true) { m_exp_opt.bIs4 = enable; }
 	inline void			set_PHTML (bool enable = true) { m_exp_opt.bIsAbiWebDoc = enable; }
 	inline void			set_MHTML (bool enable = true) { m_exp_opt.bMultipart = enable; }
-	void                addFootnote(PD_DocumentRange * pDocRange);         
-	void                addEndnote(PD_DocumentRange * pDocRange);
-	UT_sint32           getNumFootnotes(void);
-	UT_sint32           getNumEndnotes(void);
+
 private:
 	void				_buildStyleTree ();
 protected:
@@ -199,8 +200,8 @@ private:
 	s_StyleTree *		m_style_tree;
 	bool				m_bSuppressDialog;
 	XAP_Exp_HTMLOptions	m_exp_opt;
-	UT_Vector           m_vecFootnotes;
-	UT_Vector           m_vecEndnotes;
+	UT_UTF8String       m_sLinkCSS;
+	UT_UTF8String       m_sTitle;
 };
 
 #endif /* IE_EXP_HTML_H */
