@@ -26,7 +26,7 @@
 #include "ut_debugmsg.h"
 
 static const UT_uint32 merge_size_guess = 3;
-static UT_Vector m_sniffers (merge_size_guess);
+static UT_GenericVector<IE_MergeSniffer *> m_sniffers (merge_size_guess);
 
 /************************************************************************/
 /************************************************************************/
@@ -53,10 +53,8 @@ bool IE_MailMerge::fireMergeSet ()
 	
 	pDoc = m_pListener->getMergeDocument ();
 	if (pDoc) {
-		UT_StringPtrMap::UT_Cursor _hc1(&m_map);
-		for (const UT_UTF8String * _hval1 = static_cast<const UT_UTF8String *>(_hc1.first());
-			 _hc1.is_valid(); 
-			 _hval1 = static_cast<const UT_UTF8String *>(_hc1.next()) )
+		UT_GenericStringMap<UT_UTF8String *>::UT_Cursor _hc1(&m_map);
+		for (const UT_UTF8String * _hval1 = _hc1.first(); _hc1.is_valid(); _hval1 = _hc1.next() )
 		{ 
 			if (_hval1)
 				pDoc->setMailMergeField (_hc1.key(), *_hval1);
@@ -66,7 +64,7 @@ bool IE_MailMerge::fireMergeSet ()
 	}
 	
 	bool bret = m_pListener->fireUpdate ();
-	UT_HASH_PURGEDATA(UT_UTF8String*, &m_map, delete) ;
+	m_map.purgeData();
 	
 	return bret;
 }
@@ -98,7 +96,7 @@ IEMergeType IE_MailMerge::fileTypeForContents(const char * szBuf,
 	
 	for (UT_uint32 k=0; k < nrElements; k++)
 	{
-		IE_MergeSniffer * s = static_cast<IE_MergeSniffer *>(m_sniffers.getNthItem (k));
+		IE_MergeSniffer * s = m_sniffers.getNthItem (k);
 		UT_Confidence_t confidence = s->recognizeContents(szBuf, iNumbytes);
 		if ((confidence > 0) && ((IEMT_Unknown == best) || (confidence >= best_confidence)))
 		{
@@ -137,7 +135,7 @@ IEMergeType IE_MailMerge::fileTypeForSuffix(const char * szSuffix)
 	
 	for (UT_uint32 k=0; k < nrElements; k++)
 	{
-		IE_MergeSniffer * s = static_cast<IE_MergeSniffer *>(m_sniffers.getNthItem(k));
+		IE_MergeSniffer * s = m_sniffers.getNthItem(k);
 		
 		UT_Confidence_t confidence = s->recognizeSuffix(szSuffix);
 		if ((confidence > 0) && ((IEMT_Unknown == best) || (confidence >= best_confidence)))
@@ -175,7 +173,7 @@ IEMergeType IE_MailMerge::fileTypeForDescription(const char * szDescription)
 	
 	for (UT_uint32 k=0; k < nrElements; k++)
 	{
-		IE_MergeSniffer * pSniffer = static_cast<IE_MergeSniffer *>(m_sniffers.getNthItem(k));
+		IE_MergeSniffer * pSniffer = m_sniffers.getNthItem(k);
 		
 		const char * szDummy;
 		const char * szDescription2 = 0;
@@ -186,7 +184,7 @@ IEMergeType IE_MailMerge::fileTypeForDescription(const char * szDescription)
 				return ieft;
 		}
 		else
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 	}
 	
 	return ieft;
@@ -236,7 +234,7 @@ IE_MergeSniffer * IE_MailMerge::snifferForFileType(IEMergeType ieft)
 	
 	for (UT_uint32 k=0; k < nrElements; k++)
 	{
-		IE_MergeSniffer * s = static_cast<IE_MergeSniffer*>(m_sniffers.getNthItem(k));
+		IE_MergeSniffer * s = m_sniffers.getNthItem(k);
 		if (s->supportsFileType(ieft))
 			return s;
 	}
@@ -256,7 +254,7 @@ const char * IE_MailMerge::suffixesForFileType(IEMergeType ieft)
 	if (pSniffer->getDlgLabels(&szDummy,&szSuffixes,&ieftDummy))
 		return szSuffixes;
 	else
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 	
 	// The passed in filetype is invalid.
 	return 0;
@@ -273,7 +271,7 @@ const char * IE_MailMerge::descriptionForFileType(IEMergeType ieft)
 	if (pSniffer->getDlgLabels(&szDescription,&szDummy,&ieftDummy))
 		return szDescription;
 	else
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 	
 	// The passed in filetype is invalid.
 	return 0;
@@ -301,15 +299,16 @@ UT_Error IE_MailMerge::constructMerger(const char * szFilename,
 	// Give precedence to the file contents
 	if (ieft == IEMT_Unknown && szFilename && *szFilename)
 	{
-		char szBuf[4096] = "";  // 4096 ought to be enough
+		char szBuf[4097] = "";  // 4096 ought to be enough
 		UT_uint32 iNumbytes = 0;
 		FILE *f = NULL;
 		
 		// we must open in binary mode for UCS-2 compatibility
 		if ( ( f = fopen( szFilename, "rb" ) ) != static_cast<FILE *>(0) )
 		{
-			iNumbytes = fread(szBuf, 1, sizeof(szBuf), f);
+			iNumbytes = fread(szBuf, 1, sizeof(szBuf)-1, f);
 			fclose(f);
+			szBuf[iNumbytes] = '\0';
 		}
 		
 		UT_Confidence_t   best_confidence = UT_CONFIDENCE_ZILCH;
@@ -317,7 +316,7 @@ UT_Error IE_MailMerge::constructMerger(const char * szFilename,
 		
 		for (UT_uint32 k=0; k < nrElements; k++)
 		{
-		    IE_MergeSniffer * s = static_cast<IE_MergeSniffer *>(m_sniffers.getNthItem (k));
+		    IE_MergeSniffer * s = m_sniffers.getNthItem (k);
 
 		    UT_Confidence_t content_confidence = UT_CONFIDENCE_ZILCH;
 		    UT_Confidence_t suffix_confidence = UT_CONFIDENCE_ZILCH;
@@ -328,7 +327,7 @@ UT_Error IE_MailMerge::constructMerger(const char * szFilename,
 		    const char * suffix = UT_pathSuffix(szFilename) ;
 		    if ( suffix != NULL )
 			{
-				suffix_confidence = s->recognizeSuffix(UT_pathSuffix(szFilename));
+				suffix_confidence = s->recognizeSuffix(suffix);
 			}
 		    
 		    UT_Confidence_t confidence = s_confidence_heuristic ( content_confidence, 
@@ -356,7 +355,7 @@ UT_Error IE_MailMerge::constructMerger(const char * szFilename,
 	
 	for (UT_uint32 k=0; k < nrElements; k++)
 	{
-		IE_MergeSniffer * s = static_cast<IE_MergeSniffer *>(m_sniffers.getNthItem (k));
+		IE_MergeSniffer * s = m_sniffers.getNthItem (k);
 		if (s->supportsFileType(ieft))
 			return s->constructMerger(ppie);
 	}
@@ -372,7 +371,7 @@ bool IE_MailMerge::enumerateDlgLabels(UT_uint32 ndx,
 	UT_uint32 nrElements = getMergerCount();
 	if (ndx < nrElements)
     {
-		IE_MergeSniffer * s = static_cast<IE_MergeSniffer *>(m_sniffers.getNthItem (ndx));
+		IE_MergeSniffer * s = m_sniffers.getNthItem (ndx);
 		return s->getDlgLabels(pszDesc,pszSuffixList,ft);
     }
 	
@@ -409,7 +408,7 @@ void IE_MailMerge::unregisterMerger (IE_MergeSniffer * s)
 	UT_uint32 i     = 0;
 	for(i = ndx-1; i < size; i++)
     {
-		pSniffer = static_cast <IE_MergeSniffer *>(m_sniffers.getNthItem(i));
+		pSniffer = m_sniffers.getNthItem(i);
 		if (pSniffer)
 			pSniffer->setFileType(i+1);
     }
@@ -422,7 +421,7 @@ void IE_MailMerge::unregisterAllMergers ()
 	
 	for (UT_uint32 i = 0; i < size; i++)
     {
-		pSniffer = static_cast <IE_MergeSniffer *>(m_sniffers.getNthItem(i));
+		pSniffer = m_sniffers.getNthItem(i);
 		if (pSniffer)
 			pSniffer->unref();
     }
@@ -672,23 +671,23 @@ private:
 		{
 			UT_UTF8String * dup = new UT_UTF8String (item);
 			if (isHeader)
-				m_headers.addItem (static_cast<void *>(dup));
+				m_headers.addItem (dup);
 			else
-				m_items.addItem(static_cast<void *>(dup));
+				m_items.addItem(dup);
 		}
 
 	bool fire ()
 		{
 			if (m_headers.size () != m_items.size ()) {
-				UT_ASSERT(m_headers.size () == m_items.size ());
+				UT_ASSERT_HARMLESS(m_headers.size () == m_items.size ());
 				return false;
 			}
 
 			for (UT_uint32 i = 0; i < m_headers.size (); i++) {
 				UT_UTF8String * key, * val;
 
-				key = static_cast<UT_UTF8String *>(m_headers.getNthItem(i));
-				val = static_cast<UT_UTF8String *>(m_items.getNthItem(i));
+				key = m_headers.getNthItem(i);
+				val = m_items.getNthItem(i);
 
 				addMergePair (*key, *val);
 			}
@@ -699,8 +698,8 @@ private:
 			return fireMergeSet ();
 		}
 
-	UT_Vector m_headers;
-	UT_Vector m_items;
+	UT_GenericVector<UT_UTF8String *> m_headers;
+	UT_GenericVector<UT_UTF8String *> m_items;
 
 	char m_delim;
 	bool mLooping;
@@ -731,7 +730,8 @@ public:
 	}
 	
 	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix) {
-		return (!UT_stricmp (szSuffix, m_suffix.utf8_str()) ? UT_CONFIDENCE_PERFECT : UT_CONFIDENCE_POOR);
+	  // skip over "*"
+		return (!UT_stricmp (szSuffix, (m_suffix.utf8_str() + 1)) ? UT_CONFIDENCE_PERFECT : UT_CONFIDENCE_POOR);
 	}
 
 	virtual bool getDlgLabels (const char ** szDesc,
