@@ -226,13 +226,13 @@ void IE_Imp_WordPerfect::setDocumentMetaData(const UCSString &author, const UCSS
  					 const UCSString &abstract, const UCSString &descriptiveName,
 					 const UCSString &descriptiveType)
 {
-    getDoc()->setMetaDataProp(PD_META_KEY_CREATOR, UT_String(UTF8String(author).getUTF8()));
-    getDoc()->setMetaDataProp(PD_META_KEY_SUBJECT, UT_String(UTF8String(subject).getUTF8()));
-    getDoc()->setMetaDataProp(PD_META_KEY_PUBLISHER, UT_String(UTF8String(publisher).getUTF8()));
-    getDoc()->setMetaDataProp(PD_META_KEY_TYPE, UT_String(UTF8String(category).getUTF8()));
-    getDoc()->setMetaDataProp(PD_META_KEY_KEYWORDS, UT_String(UTF8String(keywords).getUTF8()));
-    getDoc()->setMetaDataProp(PD_META_KEY_LANGUAGE, UT_String(UTF8String(language).getUTF8()));
-    getDoc()->setMetaDataProp(PD_META_KEY_DESCRIPTION, UT_String(UTF8String(abstract).getUTF8()));
+    getDoc()->setMetaDataProp(PD_META_KEY_CREATOR, UTF8String(author).getUTF8());
+    getDoc()->setMetaDataProp(PD_META_KEY_SUBJECT, UTF8String(subject).getUTF8());
+    getDoc()->setMetaDataProp(PD_META_KEY_PUBLISHER, UTF8String(publisher).getUTF8());
+    getDoc()->setMetaDataProp(PD_META_KEY_TYPE, UTF8String(category).getUTF8());
+    getDoc()->setMetaDataProp(PD_META_KEY_KEYWORDS, UTF8String(keywords).getUTF8());
+    getDoc()->setMetaDataProp(PD_META_KEY_LANGUAGE, UTF8String(language).getUTF8());
+    getDoc()->setMetaDataProp(PD_META_KEY_DESCRIPTION, UTF8String(abstract).getUTF8());
 }
 
 void IE_Imp_WordPerfect::startDocument()
@@ -941,7 +941,7 @@ UT_Error IE_Imp_WordPerfect::_updateDocumentOrderedListDefinition(ABI_ListDefini
 		UT_DEBUGMSG(("AbiWordPerfect: pAuto is NULL: creating a list\n", iLevel));
 		if (iLevel > 1) 
 		{	
-			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), pListDefinition->getListID((iLevel-1)), pListDefinition->getListType(1), iStartingNumber, const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>("%L")), ".", getDoc());
+			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), pListDefinition->getListID((iLevel-1)), pListDefinition->getListType(1), iStartingNumber, const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>("%L")), ".", getDoc(), NULL);
 		}   
 		else 
 		{
@@ -952,7 +952,7 @@ UT_Error IE_Imp_WordPerfect::_updateDocumentOrderedListDefinition(ABI_ListDefini
 			sNumberingString += sTextAfterNumber.getUCS4();
 	
 			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), 0, pListDefinition->getListType(iLevel), iStartingNumber, 
-					   const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>(sNumberingString.utf8_str())), ".", getDoc());
+					   const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>(sNumberingString.utf8_str())), ".", getDoc(), NULL);
 		}
 		getDoc()->addList(pAuto);
     }
@@ -980,10 +980,10 @@ UT_Error IE_Imp_WordPerfect::_updateDocumentUnorderedListDefinition(ABI_ListDefi
 		if (iLevel > 1) 
 		{	
 			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), pListDefinition->getListID((iLevel-1)), 
-					   pListDefinition->getListType(1), 0, (XML_Char *)"%L", ".", getDoc());
+					   pListDefinition->getListType(1), 0, const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>("%L")), ".", getDoc(), NULL);
 		}   
 		else
-			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), 0, pListDefinition->getListType(iLevel), 0, (XML_Char *)"%L", ".", getDoc());
+			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), 0, pListDefinition->getListType(iLevel), 0, const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>("%L")), ".", getDoc(), NULL);
 		  
 		getDoc()->addList(pAuto);
     }
